// migrations/CreateDbExtras.ts

import { MigrationInterface, QueryRunner } from 'typeorm';

export class CreateDbExtras implements MigrationInterface {
  public async up(queryRunner: QueryRunner): Promise<void> {
    // Triggers
    await queryRunner.query(`
      DELIMITER $$

      -- Insert
      CREATE TRIGGER IF NOT EXISTS insertLifetime
      AFTER INSERT ON auctions
      FOR EACH ROW
      BEGIN
        INSERT INTO lifetimes (id, insertedon) VALUES (NEW.id, NOW(2));
      END$$

      -- Update
      CREATE TRIGGER IF NOT EXISTS updateLifetime
      AFTER UPDATE ON auctions
      FOR EACH ROW
      BEGIN
        UPDATE lifetimes SET insertedon = NOW(2) WHERE id = NEW.id;
      END$$

      -- Delete
      CREATE TRIGGER IF NOT EXISTS removeLifetime
      BEFORE DELETE ON auctions
      FOR EACH ROW
      BEGIN
        DELETE FROM lifetimes WHERE id = OLD.id;
      END$$

      DELIMITER ;
    `);

    // Stored procedure + event
    await queryRunner.query(`
      DELIMITER $$
      CREATE PROCEDURE IF NOT EXISTS CleanupOldEntries()
      BEGIN
        DECLARE done INT DEFAULT 0;
        DECLARE temp_id INT;

        DECLARE cur CURSOR FOR 
          SELECT id
          FROM lifetimes 
          WHERE TIMESTAMPDIFF(HOUR, insertedon, NOW()) >= 6;
        DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;

        OPEN cur;
        read_loop: LOOP
          FETCH cur INTO temp_id;
          IF done THEN
            LEAVE read_loop;
          END IF;
          DELETE FROM auctions WHERE id = temp_id;
        END LOOP;
        CLOSE cur;
      END$$
      DELIMITER ;

      SET GLOBAL event_scheduler = ON;

      CREATE EVENT IF NOT EXISTS CleanupOldEntriesEvent
      ON SCHEDULE EVERY 2 MINUTE
      DO
        CALL CleanupOldEntries();
    `);
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    // Reverse: drop triggers, event, procedure, etc.
    await queryRunner.query(`DROP EVENT IF EXISTS CleanupOldEntriesEvent;`);
    await queryRunner.query(`DROP PROCEDURE IF EXISTS CleanupOldEntries;`);
    await queryRunner.query(`DROP TRIGGER IF EXISTS removeLifetime;`);
    await queryRunner.query(`DROP TRIGGER IF EXISTS updateLifetime;`);
    await queryRunner.query(`DROP TRIGGER IF EXISTS insertLifetime;`);
  }
}
